import argparse
import tempfile
from pathlib import Path
from typing import Dict, List, Tuple

import speech_recognition as sr  # type: ignore
from pydub import AudioSegment  # type: ignore
from vst.classes.languages import LanguageToLanguageTag  # type: ignore
from vst.classes.shell import Shell  # type: ignore


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-i', '--input', action='store', required=True, type=Path,
        help='Path to folder with WAVs files',
    )
    parser.add_argument(
        '-l', '--language', action='store', required=False, type=str, default='Polish',
        help='Language name. Read README.md file to know all languages',
    )
    parser.add_argument(
        '-o', '--output', action='store', required=True, type=Path,
        help='Path to the folder where training and validation files will be saved',
    )
    parser.add_argument(
        '-p', '--percent', action='store', required=False, type=int, default=0,
        help='Path to folder with WAVs files',
    )
    parser.add_argument(
        '-r', '--recursive', action='store_true', required=False,
        help='Find files also in subdirectories of input path',
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true', required=False,
        help='Show information while the script is running',
    )
    return parser.parse_args()


def split_files_for_training_and_validation(files: List[Path], percent: int) -> Tuple[List[Path], List[Path]]:
    list_of_files = list(files)
    number_of_training_files = int((len(list_of_files) / 100) * percent + 0.5)
    number_of_validation_files = number_of_training_files if number_of_training_files > 2 else 2
    return list_of_files[number_of_validation_files:len(list_of_files)], list_of_files[:number_of_validation_files]


def get_text_from_wav_file(path: Path, language: str) -> str:
    r = sr.Recognizer()
    language_tag = str(LanguageToLanguageTag(language))
    silence = AudioSegment.silent(duration=1000)
    wav_content = AudioSegment.from_wav(path)
    new_content = silence + wav_content + silence
    path_to_temporary_file = Path(tempfile.gettempdir()) / 'temporary_wav_file.wav'
    new_content.export(path_to_temporary_file, format='wav')

    with sr.AudioFile(str(path_to_temporary_file)) as file:
        audio = r.record(file)
        try:
            text = r.recognize_google(audio, language=language_tag)
        except sr.UnknownValueError:
            text = f"<< Text wasn't recognized! Are you sure that the language used in the wav file is {language} ? >>"

    path_to_temporary_file.unlink()
    return text


def get_dialogs_from_files(paths: List[Path], language: str, verbose: bool = False) -> Dict[Path, str]:
    dialogs = {}
    for index, path in enumerate(paths, start=1):
        text = get_text_from_wav_file(Path(path), language)
        if verbose:
            print(f'{index:5}: {Path(path).name:50} {text}')
        dialogs[Path(path)] = text
    return dialogs


def convert_dialogs_to_file_content(dialogs: Dict[Path, str]) -> str:
    content = ''
    for dialog in dialogs:
        content += f'{dialog}|{dialogs[dialog]}.\n'
    return content


def save_content_to_file(path: Path, content: str) -> None:
    with open(path, 'w', encoding='utf8') as file:
        file.write(content)


def wav_to_text() -> None:
    arguments = parse_arguments()

    shell = Shell()
    wav_files_paths = shell.get_files_by_extensions(arguments.input, ['wav'], arguments.recursive)

    if arguments.percent == 0:
        training_files = wav_files_paths
        validation_files = []
    elif 0 < arguments.percent < 100:
        training_files, validation_files = split_files_for_training_and_validation(wav_files_paths, arguments.percent)
    else:
        print('The value of param "percent" should be between 0 and 99 percent')
        exit(2)

    operations = {
        'training': training_files,
        'validation': validation_files,
    }

    if arguments.verbose:
        message = f'''
        \r    Number of wav files: {len(training_files) + len(validation_files)}
        \r         Training files: {len(training_files)}
        \r       Validation files: {len(validation_files)}
        \r         Using language: {LanguageToLanguageTag(arguments.language).selected_language}
        \r              Recursive: {'Yes' if arguments.recursive else 'No'}'''
        print(message)

    if not Path(arguments.output).exists():
        shell.mkdir(arguments.output, parent=True)

    for operation in operations:
        files = operations[operation]
        if len(files) > 0:
            if arguments.verbose:
                print(f'\n => Converting {operation} files ({len(files)} files)')
            dialogs = get_dialogs_from_files(files, arguments.language, arguments.verbose)
            file_content = convert_dialogs_to_file_content(dialogs)
            save_content_to_file(Path(arguments.output) / f'{operation}.txt', file_content)


def main() -> None:
    try:
        wav_to_text()
    except KeyboardInterrupt:
        print(' <= Application terminated by the user\n')
        exit(0)


if __name__ == '__main__':
    main()



import os
import shutil
from contextlib import contextmanager
from pathlib import Path

from vst.classes.process import Process, ProcessResult  # type: ignore

_PROCESS_FAILED = 'Shell command failed with exit code: {} \nError message: \n{}.'


class ShellProcessFailedError(Exception):
    def __init__(self, exit_code: int, error_message: str):
        super().__init__(_PROCESS_FAILED.format(exit_code, error_message))


class Shell(Process):

    def run(self, command: str) -> ProcessResult:
        shell_results = self._run(f'bash -c -i {command}')

        if shell_results.exit_code:
            raise ShellProcessFailedError(shell_results.exit_code, shell_results.error_message)

        return shell_results

    @staticmethod
    @contextmanager
    def cd(path: str) -> None:
        start_path = os.getcwd()
        try:
            os.chdir(path)
            yield
        finally:
            os.chdir(start_path)

    @staticmethod
    def mkdir(path: str, parent: bool = False) -> None:
        if parent:
            os.makedirs(path)
        else:
            os.mkdir(path)

    @staticmethod
    def move(source_path: str, destination_path: str) -> None:
        shutil.move(source_path, destination_path)

    @staticmethod
    def copy(source_path: str, destination_path: str) -> None:
        shutil.copy(source_path, destination_path)

    @staticmethod
    def rmtree(path: str) -> None:
        try:
            shutil.rmtree(path)
        except FileNotFoundError:
            pass

    @staticmethod
    def open(path: str) -> str:
        with open(path) as file:
            return file.read()

    @staticmethod
    def save(path: str, content: str) -> None:
        with open(path, 'w') as file:
            file.write(content)

    @staticmethod
    def get_files_by_extensions(path: str, extensions: str, recursive: bool = False) -> list:
        pattern = '**/*' if recursive else '*'
        return sum([sorted(Path(path).glob(f'{pattern}.{extension}')) for extension in extensions], [])



import argparse
from pathlib import Path

import ffmpeg  # type: ignore
from vst.classes.shell import Shell  # type: ignore


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-i', '--input', action='store', required=True, type=Path,
        help='Path to folder with audio files (ogg/mp3/wav)',
    )

    parser.add_argument(
        '-o', '--output', action='store', required=True, type=Path,
        help='Set destination path to folder where to save WAV files',
    )
    parser.add_argument(
        '-t', '--truncate', action='store_true', required=False,
        help='Truncate silence',
    )
    parser.add_argument(
        '-r', '--recursive', action='store_true', required=False,
        help='Find files also in subdirectories of input path',
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true', required=False,
        help='Show information while the script is running',
    )
    return parser.parse_args()


def audio_to_wav():
    arguments = parse_arguments()
    shell = Shell()

    supported_audio_formats = [
        'aac', 'ac3', 'aiff', 'ape', 'flac', 'm4a', 'mp2', 'mp3', 'mp4', 'ogg', 'ogx', 'opus', 'ts', 'wav', 'wma', 'wv',
    ]

    audio_files_paths = shell.get_files_by_extensions(arguments.input, supported_audio_formats, arguments.recursive)
    number_of_all_files = len(audio_files_paths)

    if arguments.verbose:
        message = f'''
        \r  Number of audio files: {number_of_all_files}
        \r             Input path: {arguments.input}
        \r            Output path: {arguments.output}
        \r              Recursive: {'Yes' if arguments.recursive else 'No'}
        \r               Truncate: {'Yes' if arguments.truncate else 'No'}
        '''
        print(message)

    if not Path(arguments.output).exists():
        shell.mkdir(arguments.output, parent=True)

    for index, path in enumerate(audio_files_paths, start=1):
        relative_path_to_file = Path(str(path).replace(str(arguments.input), '').strip('/')).with_suffix('.wav')
        full_path_to_file = arguments.output / relative_path_to_file
        folder_to_create = full_path_to_file.parent

        if not folder_to_create.exists():
            shell.mkdir(folder_to_create, parent=True)

        if arguments.verbose:
            print(f'Converting file: {index}/{number_of_all_files} ({path})')

        audio_stream = ffmpeg.output(
            ffmpeg.input(path),
            str(full_path_to_file),
            format='wav',
            acodec='pcm_s16le',
            ac=1,
            ar='22050',
        )
        ffmpeg.run(audio_stream, overwrite_output=True, capture_stderr=True, capture_stdout=True)

        if arguments.truncate:
            audio_stream = ffmpeg.output(
                ffmpeg.input(full_path_to_file),
                str(full_path_to_file.with_suffix('.TS.wav')),
                af='silenceremove'
                   '=start_periods=1'
                   ':stop_duration=0.4'
                   ':start_threshold=-50dB'
                   ':stop_periods=-1'
                   ':stop_duration=0.4'
                   ':stop_threshold=-50dB',
            )
            ffmpeg.run(audio_stream, overwrite_output=True, capture_stderr=True, capture_stdout=True)


def main() -> None:
    try:
        audio_to_wav()
    except KeyboardInterrupt:
        print(' <= Application terminated by the user\n')
        exit(0)


if __name__ == '__main__':
    main()



import subprocess
from typing import NamedTuple

_TIMEOUT_MESSAGE = 'Command "{}" timed out.'
_SHELL_TIMEOUT = 1000


class ProcessResult(NamedTuple):
    stdout: str
    exit_code: int
    error_message: str


class ProcessTimeoutError(Exception):
    def __init__(self, command: str):
        super().__init__(_TIMEOUT_MESSAGE.format(command))


class Process:  # pylint: disable=too-few-public-methods
    __slots__ = ('_shell_process', )

    _shell_process: subprocess.Popen

    def _run(self, command: str) -> ProcessResult:
        self._shell_process = subprocess.Popen(
            command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, universal_newlines=True,
        )

        try:
            stdout, stderr = self._shell_process.communicate(timeout=_SHELL_TIMEOUT)
        except TimeoutError:
            self._shell_process.kill()
            raise ProcessTimeoutError(command)

        return ProcessResult(stdout=stdout, exit_code=self._shell_process.returncode, error_message=stderr)



import argparse
from pathlib import Path

import requests
from phonemizer.phonemize import phonemize

from vst.classes.shell import Shell  # type: ignore

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
# Code used from ForwardTacotron because ForwardTacotron is not a library :(
# https://github.com/as-ideas/ForwardTacotron/blob/master/utils/text/cleaners.py
# https://github.com/as-ideas/ForwardTacotron/blob/master/utils/text/symbols.py

_pad = '_'
_punctuation = '!\'(),.:;? '
_special = '-'

# Phonemes
_vowels = 'iyɨʉɯuɪʏʊeøɘəɵɤoɛœɜɞʌɔæɐaɶɑɒᵻ'
_non_pulmonic_consonants = 'ʘɓǀɗǃʄǂɠǁʛ'
_pulmonic_consonants = 'pbtdʈɖcɟkɡqɢʔɴŋɲɳnɱmʙrʀⱱɾɽɸβfvθðszʃʒʂʐçʝxɣχʁħʕhɦɬɮʋɹɻjɰlɭʎʟ'
_suprasegmentals = 'ˈˌːˑ'
_other_symbols = 'ʍwɥʜʢʡɕʑɺɧ'
_diacrilics = 'ɚ˞ɫ'
# some extra symbols that I found in from wiktionary ipa annotations
_extra_phons = ['g', 'ɝ', '̃', '̍', '̥', '̩', '̯', '͡']

phonemes = list(
    _pad + _punctuation + _special + _vowels + _non_pulmonic_consonants
    + _pulmonic_consonants + _suprasegmentals + _other_symbols + _diacrilics) + _extra_phons

phonemes_set = set(phonemes)


def to_phonemes(text: str, lang: str) -> str:
    phonemes = phonemize(text,
                         language=lang,
                         backend='espeak',
                         strip=True,
                         preserve_punctuation=True,
                         with_stress=False,
                         njobs=1,
                         punctuation_marks=';:,.!?¡¿—…"«»“”()',
                         language_switch='remove-flags')
    phonemes = ''.join([p for p in phonemes if p in phonemes_set])
    return phonemes
# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =


def translator(translation, sentence: str, accent: str) -> str:
    new_sentence = sentence
    for key, value in translation.items():
        new_sentence = new_sentence.replace(key, value.get(accent, value.get('general', key)))
    return new_sentence


def download_translation(download_path: Path, url_path: str = 'default'):
    package_name = 'vst'
    url = url_path

    if url_path == 'default':
        url = f'https://raw.githubusercontent.com/8tm/{package_name}/master/src/{package_name}/translation/en_to_pl.py'

    r = requests.get(url, allow_redirects=True)

    shell = Shell()
    shell.rmtree(str(download_path))
    shell.mkdir(path=str(download_path))

    with open(f'{download_path}/downloaded_en_to_pl.py', 'w') as translation_file:
        translation_file.write(r.content.decode())


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-i', '--input', action='store', required=True, type=Path,
        help='Path to english training/validation file',
    )
    parser.add_argument(
        '-o', '--output', action='store', required=True, type=Path,
        help='Path to the folder where new translated training/validation files will be saved',
    )
    parser.add_argument(
        '-d', '--dialect', action='store', required=False, type=str, default='en-us',
        help='Select dialect. Default: en-us',
    )
    parser.add_argument(
        '-t', '--translation', action='store', required=False, type=str, default='default',
        help='URL to translation file en_to_pl.py or use default one if parametr is empty or not set.',
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true', required=False,
        help='Show information while the script is running',
    )
    return parser.parse_args()


def eng_to_pl():
    arguments = parse_arguments()

    if not Path(arguments.input).exists():
        print(f'File not found: {arguments.input}')
        exit(1)

    if arguments.translation != 'default':
        print(f'Downloading translation from: {arguments.translation}')
        download_path = Path('/tmp/vst/')
        download_translation(download_path, arguments.translation)
        import sys
        sys.path.append(str(download_path))
        from downloaded_en_to_pl import translation
    else:
        from vst.translation.en_to_pl import translation

    with open(arguments.input, 'r') as transcode_file:
        lines = transcode_file.readlines()

    translated_lines = []

    for index, line in enumerate(lines):
        elements = line.split('|')
        ipa_text = to_phonemes(elements[1], arguments.dialect)
        elements[1] = translator(translation, ipa_text, arguments.dialect)
        new_line = '|'.join(element.replace('\n', '') for element in elements) + '\n'
        translated_lines.append(new_line)

        if arguments.verbose:
            print(new_line)

    with open(arguments.output, 'w') as new_transcode_file:
        new_transcode_file.writelines(translated_lines)


def main():
    try:
        eng_to_pl()
    except KeyboardInterrupt:
        print(' <= Application terminated by the user\n')
        exit(0)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['vst', 'vst.classes', 'vst.translation']

package_data = \
{'': ['*']}

install_requires = \
['SpeechRecognition>=3.8.1,<4.0.0',
 'ffmpeg-python>=0.2.0,<0.3.0',
 'phonemizer>=3.2.1,<4.0.0',
 'pydub>=0.25.1,<0.26.0',
 'requests>=2.28.0,<3.0.0']

entry_points = \
{'console_scripts': ['vst_audio_to_wav = vst.audio_to_wav:main',
                     'vst_en_to_pl = vst.en_to_pl:main',
                     'vst_wav_to_text = vst.wav_to_text:main']}

setup_kwargs = {
    'name': 'vst',
    'version': '22.6.25',
    'description': 'VST - Voice Simple Tools',
    'long_description': '# VST (Voice Simple Tools)\n\nPlease check [WIKI](https://github.com/8tm/vst/wiki) pages for project description.\n',
    'author': 'Tadeusz Miszczyk',
    'author_email': '42252259+8tm@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://github.com/8tm/vst',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.13,<4.0.0',
}


setup(**setup_kwargs)



class LanguageToLanguageTag:
    default_language: str = 'Polish'
    languages: dict = {
        'English': 'en-GB',
        'German': 'de-DE',
        'Polish': 'pl-PL',
    }
    selected_language: str = ''
    selected_tag: str = ''

    def __init__(self, language_name: str) -> None:
        self.selected_language = language_name if language_name in self.__class__.languages.keys() \
            else self.default_language

    def __str__(self) -> str:
        self.selected_tag = str(self.__class__.languages[self.selected_language])
        return self.selected_tag
